from re import T
import time

from flask import app

from citybrain_platform import JobStatus
import citybrain_platform
from citybrain_platform import Column, ColumnType
from citybrain_platform.computing import data_types
from citybrain_platform.computing.data_types import ExternalFiletype, ResourceType


citybrain_platform.api_key = "MiQxNjkyMTU1Nzk3JGJxaXplaXRu"
citybrain_platform.api_baseurl = "http://127.0.0.1:8080/platform/"

# result = citybrain_platform.Computing.create_resource(
#     name="add.py",
#     type=ResourceType.PY,
#     comment="for add",
#     local_file="udf.py"
# )
# print(result)


# result = citybrain_platform.Computing.create_function(
#     name="test_udf_add3",
#     class_file="add.py",
#     class_name="MyPlus",
#     comment="test add function",
#     resources=["add.py"]
# )
# print(result)

# citybrain_platform.Computing.update_table_status(name="mbt_test1130", public=True)


# upload parquet file to storage service
# res = citybrain_platform.Storage.upload_file(
#     remote_path="tbl_test_cmip/user_storage_1_cmiptest_cmip6_ta_Amon_IITM-ESM_ssp585_r1i1p1f1_gn_208501-209412.nc.gzip", 
#     local_file="user_storage_1_cmiptest_cmip6_ta_Amon_IITM-ESM_ssp585_r1i1p1f1_gn_208501-209412.nc.gzip"
# )
# print(res)

# create table using files from storage service
# columns = [
#     Column("cfc113global", ColumnType.DOUBLE, ""),
#     Column("time_bnds", ColumnType.TIMESTAMP, ""),
#     Column("time", ColumnType.TIMESTAMP, ""),
#     Column("bnds", ColumnType.DOUBLE, "")
# ]

# res = citybrain_platform.Computing.create_table(
#     name="tbl_test_cmip", 
#     columns=columns,
#     description="test table",
#     storage_filesource="/tbl_test_cmip",
#     storage_filetype=data_types.ExternalFiletype.PARQUET
# );
# print(res);

# job_id = citybrain_platform.Computing.create_job("select test_udf_add3(12, 14);")
# print(job_id);

# # make a select query
# status = citybrain_platform.Computing.get_job_status(job_id='99c1ce12-6a45-48f8-9685-6115093bfaff');
# print(status);

# # get results of the query
citybrain_platform.Computing.get_job_results(job_id='99c1ce12-6a45-48f8-9685-6115093bfaff', filepath="tbl_test_cmip21.csv", header=True)


# res = citybrain_platform.Storage.delete_file(remote_path="fawda/fsaw/a.csv")
# print(res)

# res = citybrain_platform.Storage.list_files(prefix="", direct_only=False)
# print(res)

# res = citybrain_platform.Storage.upload_file(remote_path="testaa/a.csv", local_file="a.csv")
# print(res)


# citybrain_platform.Storage.download_file(remote_path="fawda/faw/a.csv", local_file="b.csv")


columns = [
    Column("id1", ColumnType.INT, "aa"),
    Column("id2", ColumnType.INT, "bb"),
]

# create table
# ok = citybrain_platform.Computing.create_table(name="mbt_testaaaa", columns=columns, description="ssdad")
# print(ok)

# result = citybrain_platform.Computing.upload_table_data(name="mbt_testaaa", append=True, csv_filepath="a.csv")
# print(result)

# partition_columns = [
#     Column("col_pt", ColumnType.INT, "ppt")
# ]

# job_id=citybrain_platform.Computing.create_job("select * from mbt_testaaa limit 12;")
# print(job_id)
# print(citybrain_platform.Computing.get_job_status(job_id='18bf62d1-d9c9-41cd-8e74-a1f064381e89'))
# citybrain_platform.Computing.get_job_results(job_id='18bf62d1-d9c9-41cd-8e74-a1f064381e89', filepath="testaa.csv")

# columns = [
#     Column("time", ColumnType.TIMESTAMP, ""),
#     Column("bnds", ColumnType.BIGINT, ""),
#     Column("plev", ColumnType.DOUBLE, ""),
#     Column("lat", ColumnType.DOUBLE, ""),
#     Column("lon", ColumnType.DOUBLE, ""),
#     Column("time_bnds", ColumnType.TIMESTAMP, ""),
#     Column("lat_bnds", ColumnType.DOUBLE, ""),
#     Column("lon_bnds", ColumnType.DOUBLE, ""),
#     Column("ta", ColumnType.DOUBLE, ""),
# ]


# create table
# ok = citybrain_platform.Computing.create_table(name="mbt_testcmip6", columns=columns, description="ssdad", storage_filesource="cmiptest/", storage_filetype=ExternalFiletype.PARQUET)
# print(ok)

# get table schema
# schema = citybrain_platform.Computing.get_table_schema(name="mbt_test4")
# print(schema)

# upload data to table
# result = citybrain_platform.Computing.upload_table_data(name="mbt_test2", append=True, csv_filepath="aa.csv", partition_key={"col_pt": "19"})
# print(result)

# truncate table
# result = citybrain_platform.Computing.truncate_table(name="mbt_test1", partition_key={"col_pt": "1"})
# print(result)

# drop table
# result = citybrain_platform.Computing.drop_table(name="mbt_test2")
# print(result)

# public table
# public_table_name = citybrain_platform.Computing.public_table(name="mbt_test1")
# print(public_table_name)

# get available table list
# result = citybrain_platform.Computing.list_tables()
# print(result)

# create sql job
# job_id = citybrain_platform.Computing.create_job(
#     sql="select osmid from osm_node where osmid between 80080000 and 80081000;",
#     worker_limit=100,
#     split_size=256
# )
# print(job_id)

# stop running job
# result = citybrain_platform.Computing.stop_job(job_id="fab7b329-aef9-4c96-a9ac-1b47b1468e79")
# print(result)

# get job status
# job_status = citybrain_platform.Computing.get_job_status(job_id="e5597f65-b12a-4c3f-bb36-6cd742ec9b44")
# print(job_status)


# job_id = citybrain_platform.Computing.create_job(sql="select * from eric.zhang.city_Oklahoma limit 10;")
# print(job_id)

# get job results
# citybrain_platform.Computing.get_job_results(job_id="dc9132b8-8697-4c1e-9e77-796eb0ec7584", filepath="a.csv")


# sqlresult = "select * from cesm_ass1_50year_0730 limit 1000"
# jobresult_id = citybrain_platform.Computing.create_job(sql=sqlresult)

# while True:
#     status = citybrain_platform.Computing.get_job_status(job_id=jobresult_id)
#     print(status.status)
#     if status.status == JobStatus.RUNNING:
#         print(status.progress)
#     if status.status == JobStatus.TERMINATED:
#         break
#     time.sleep(1)

# print("downloading result")
# citybrain_platform.Computing.get_job_results(job_id=jobresult_id, filepath="cesm_ass1_50year_0730.csv")

# citybrain_platform.Data.download(data_address="174B948F17421000", save_file="a.txt")



__all__ = ["Data"]

import citybrain_platform
from citybrain_platform.data.client import DataClient


class Data():
    __CLIENT: DataClient = None

    @classmethod
    def __client(cls) -> DataClient:
        if getattr(cls, '__CLIENT', None) is None:
            cls.__CLIENT = DataClient(base_url=citybrain_platform.api_baseurl, api_key=citybrain_platform.api_key)
        return cls.__CLIENT
    
    @classmethod
    def download(cls, data_address: str, save_file: str):
        resp = cls.__client().download(data_address=data_address)
        resp.raise_for_status()
        
        with open(save_file, 'wb') as f:
            for chunk in resp.iter_content(chunk_size=4096): 
                f.write(chunk)
    
    @classmethod
    def add_remote(cls, name: str, description: str, remote_url: str) -> str:
        result = cls.__client().add_remote(name=name, description=description, url=remote_url)
        return result

    @classmethod
    def add_storage(cls, name: str, description: str, file_path: str) -> str:
        result = cls.__client().add_storage(name=name, description=description, file_path=file_path)
        return result


        


from uplink import Body, Consumer, Field, Query, get, inject, json, post, response_handler
from uplink.hooks import TransactionHook

class APIResponseException(Exception):
    pass

class ResponseStreamHook(TransactionHook):
    def audit_request(self, consumer, request_builder):
        request_builder.info["stream"] = True
        return super().audit_request(consumer, request_builder)

_response_stream_hook = ResponseStreamHook()

def handle_json_response(response):
    response.iter_lines()
    if response.headers["Content-Type"].startswith("application/json"):
        body = response.json()
        if body["code"] != 200:
            raise APIResponseException(body["message"])
        return body["data"]
    return response

@response_handler(handle_json_response)
class StorageClient(Consumer):
    """A Python Client for the Citybrain Storage Platform API."""

    def __init__(self, base_url, api_key):
        super(StorageClient, self).__init__(base_url=base_url)
        self.session.headers["api_key"] = api_key

    @get("storage/file/list")
    def file_list(self, prefix: Query("prefix"), direct_only: Query("direct_only")):
        pass

    @post("storage/file/upload")
    def file_upload(self, path: Query("path"), body: Body):
        pass

    @json
    @post("storage/file/delete")
    def file_delete(self, path: Field("path")):
        pass

    @inject(_response_stream_hook)
    @get("storage/file/download")
    def file_download(self, path: Query("path")):
        pass


from dataclasses import  asdict
import json
from typing import Dict, List
from dacite import from_dict, Config
from citybrain_platform.computing import data_types
from citybrain_platform.computing.client import ComputingClient
import citybrain_platform

__all__ = ["Computing"]

class Computing():
    __CLIENT: ComputingClient = None

    @classmethod
    def __client(cls) -> ComputingClient:
        if getattr(cls, '__CLIENT', None) is None:
            cls.__CLIENT = ComputingClient(base_url=citybrain_platform.api_baseurl, api_key=citybrain_platform.api_key)
        return cls.__CLIENT
    
    @classmethod
    def create_table(cls, name: str, columns: List[data_types.Column], partition_columns: List[data_types.Column] = [], description: str = "", storage_filesource: str = "", storage_filetype: data_types.ExternalFiletype = data_types.ExternalFiletype.CSV) -> bool:
        result = cls.__client().table_create(
            name=name,
            columns=[asdict(col) for col in columns],
            partition_columns=[asdict(col) for col in partition_columns],
            description=description,
            storage_filesource=storage_filesource,
            storage_filetype=storage_filetype.value
        )
        return result

    @classmethod
    def get_table_schema(cls, name: str) -> data_types.Schema:
        result = cls.__client().table_info(name=name)
        table_schema = from_dict(
            data_class=data_types.Schema, 
            data=result,
            config=Config(type_hooks={data_types.ColumnType: data_types.ColumnType}),
        )
        return table_schema
    
    @classmethod
    def upload_table_data(cls, name: str, append: bool, csv_filepath: str, partition_key: Dict[str, str] = None) -> bool:
        appendParam = "true" if append else "false"
        result = False
        with open(csv_filepath, 'rb') as f:
            result = cls.__client().table_upload(name=name, append=appendParam, partition_key=json.dumps(partition_key), body=f)
        return result
    
    @classmethod
    def truncate_table(cls, name: str, partition_key: Dict[str, str] = None) -> bool:
        result = cls.__client().table_truncate(name=name, partition_key=partition_key)
        return result
    
    @classmethod
    def drop_table(cls, name: str) -> bool:
        result = cls.__client().table_drop(name=name)
        return result
    
    @classmethod
    def update_table_status(cls, name: str, public: bool) -> str:
        status = "public" if public else "private"
        result = cls.__client().update_table_status(name=name, status=status)
        return result

    
    @classmethod
    def list_tables(cls) -> data_types.AvaliableTableList:
        result = cls.__client().table_list()
        table_list = from_dict(data_class=data_types.AvaliableTableList, data=result)
        return table_list
    
    @classmethod
    def create_function(cls, name: str, class_file: str, class_name: str, resources: List[str], comment: str = "") -> bool:
        result = cls.__client().create_function(name=name, class_file=class_file, class_name=class_name, comment=comment, resources=resources)
        return result
    
    @classmethod
    def create_resource(cls, name: str, type: data_types.ResourceType, local_file: str, comment: str = "") -> bool:
        result = False
        with open(local_file, "rb") as f:
            result = cls.__client().create_resource(name=name, type=type.value, comment=comment, body=f)
        return result
    
    @classmethod
    def create_job(cls, sql: str, worker_limit: int = 0, split_size: int = 0) -> str:
        result = cls.__client().job_submit(sql=sql, worker_limit=worker_limit, split_size=split_size)
        return result
    
    @classmethod
    def stop_job(cls, job_id: str) -> bool:
        result = cls.__client().job_stop(job_id=job_id)
        return result
    
    @classmethod
    def get_job_status(cls, job_id: str) -> data_types.JobStatusInfo:
        result = cls.__client().job_status(job_id=job_id)
        job_status = from_dict(
            data_class=data_types.JobStatusInfo, 
            data=result, 
            config=Config(type_hooks={data_types.JobStatus: data_types.JobStatus}),
        )

        if job_status.summary != None:
            job_status.summary.cpu_cost /= 100
            job_status.summary.mem_cost /= 100

        return job_status
    
    @classmethod
    def get_job_results(cls, job_id: str, filepath: str, append: bool = False, header: bool = True):
        response = cls.__client().job_result(job_id=job_id, header=header)
        with open(filepath, "ab" if append else "wb") as f:
            for l in response.iter_lines():
                f.write(l)
                f.write(b'\n')



from uplink import Consumer, Field, Query, get, inject, json, post, response_handler
from uplink.hooks import TransactionHook

class ResponseStreamHook(TransactionHook):
    def audit_request(self, consumer, request_builder):
        request_builder.info["stream"] = True
        return super().audit_request(consumer, request_builder)

_response_stream_hook = ResponseStreamHook()

class APIResponseException(Exception):
    pass

def handle_json_response(response):
    response.iter_lines()
    if response.headers["Content-Type"].startswith("application/json"):
        body = response.json()
        if body["code"] != 200:
            raise APIResponseException(body["message"])
        return body["data"]
    return response

@response_handler(handle_json_response)
class DataClient(Consumer):
    """A Python Client for the Citybrain Data Platform API."""

    def __init__(self, base_url, api_key):
        super(DataClient, self).__init__(base_url=base_url)
        self.session.headers["api_key"] = api_key

    @inject(_response_stream_hook)
    @get("data/download")
    def download(self, data_address: Query("data_address")):
        pass

    @json
    @post("data/add_remote")
    def add_remote(self, name: Field("name"), description: Field("description"), url: Field("url")):
        pass

    @json
    @post("data/add_storage")
    def add_storage(self, name: Field("name"), description: Field("description"), file_path: Field("file_path")):
        pass